@@ -281,7 +281,7 @@ xfdesktop_thumbnailer_is_supported(XfdesktopThumbnailer *thumbnailer,
     mime_type = xfdesktop_get_file_mimetype(file);
 
     if(mime_type == NULL) {
-        DBG("File %s has no mime type", file);
+        XF_DEBUG("File %s has no mime type", file);
         return FALSE;
     }
 
@@ -315,7 +315,7 @@ xfdesktop_thumbnailer_queue_thumbnail(XfdesktopThumbnailer *thumbnailer,
     g_return_val_if_fail(file != NULL, FALSE);
 
     if(!xfdesktop_thumbnailer_is_supported(thumbnailer, file)) {
-        DBG("file: %s not supported", file);
+        XF_DEBUG("file: %s not supported", file);
         return FALSE;
     }
     if(thumbnailer->priv->request_timer_id) {
@@ -330,8 +330,8 @@ xfdesktop_thumbnailer_queue_thumbnail(XfdesktopThumbnailer *thumbnailer,
             {
                 /* If this fails it usually means there's a thumbnail already
                  * being processed, no big deal */
-                DBG("Dequeue of thumbnailer->priv->handle: %d failed",
-                    thumbnailer->priv->handle);
+                XF_DEBUG("Dequeue of thumbnailer->priv->handle: %d failed",
+                         thumbnailer->priv->handle);
             }
 
             thumbnailer->priv->handle = 0;
@@ -387,8 +387,8 @@ xfdesktop_thumbnailer_dequeue_thumbnail(XfdesktopThumbnailer *thumbnailer,
             {
                 /* If this fails it usually means there's a thumbnail already
                  * being processed, no big deal */
-                DBG("Dequeue of thumbnailer->priv->handle: %d failed",
-                    thumbnailer->priv->handle);
+                XF_DEBUG("Dequeue of thumbnailer->priv->handle: %d failed",
+                         thumbnailer->priv->handle);
             }
         }
         thumbnailer->priv->handle = 0;
@@ -561,15 +561,17 @@ xfdesktop_thumbnailer_thumbnail_ready_dbus(DBusGProxy *proxy,
                                                   filename, NULL);
             }
 
-            DBG("thumbnail-ready src: %s thumbnail: %s",
-                (char*)iter->data,
-                thumbnail_location);
+            XF_DEBUG("thumbnail-ready src: %s thumbnail: %s",
+                     (char*)iter->data,
+                     thumbnail_location);
 
-            g_signal_emit(G_OBJECT(thumbnailer),
-                          thumbnailer_signals[THUMBNAIL_READY],
-                          0,
-                          iter->data,
-                          thumbnail_location);
+            if(g_file_test(thumbnail_location, G_FILE_TEST_EXISTS)) {
+                g_signal_emit(G_OBJECT(thumbnailer),
+                              thumbnailer_signals[THUMBNAIL_READY],
+                              0,
+                              iter->data,
+                              thumbnail_location);
+            }
 
             temp = iter->data;
             thumbnailer->priv->queue = g_slist_remove(thumbnailer->priv->queue,
