@@ -487,7 +487,7 @@ xfce_backdrop_choose_chronological(XfceBackdrop *backdrop)
     /* Figure out which image to display based on what time of day it is
      * and how many images we have to work with */
     epoch = (gdouble)g_date_time_get_hour(datetime) / (24.0f / MIN(n_items, 24.0f));
-    DBG("epoch %d, hour %d, items %d", epoch, g_date_time_get_hour(datetime), n_items);
+    XF_DEBUG("epoch %d, hour %d, items %d", epoch, g_date_time_get_hour(datetime), n_items);
 
     new_file = g_list_nth(backdrop->priv->image_files, epoch);
 
@@ -1147,7 +1147,7 @@ xfce_backdrop_timer(XfceBackdrop *backdrop)
         /* remove old timer first */
         xfce_backdrop_remove_backdrop_timer(backdrop);
 
-        DBG("calling g_timeout_add_seconds, interval is %d", cycle_interval);
+        XF_DEBUG("calling g_timeout_add_seconds, interval is %d", cycle_interval);
         backdrop->priv->cycle_timer_id = g_timeout_add_seconds(cycle_interval,
                                                                (GSourceFunc)xfce_backdrop_timer,
                                                                backdrop);
@@ -1239,7 +1239,7 @@ xfce_backdrop_set_cycle_timer(XfceBackdrop *backdrop, guint cycle_timer)
             }
 
         if(cycle_interval != 0) {
-            DBG("calling g_timeout_add_seconds, interval is %d", cycle_interval);
+            XF_DEBUG("calling g_timeout_add_seconds, interval is %d", cycle_interval);
             backdrop->priv->cycle_timer_id = g_timeout_add_seconds(cycle_interval,
                                                                    (GSourceFunc)xfce_backdrop_timer,
                                                                    backdrop);
@@ -1358,6 +1358,20 @@ xfce_backdrop_get_random_order(XfceBackdrop *backdrop)
     return backdrop->priv->random_backdrop_order;
 }
 
+void
+xfce_backdrop_force_cycle(XfceBackdrop *backdrop)
+{
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+
+    TRACE("entering");
+
+    /* force it to update */
+    xfce_backdrop_cycle_backdrop(backdrop);
+
+    /* Update the timer, if running */
+    xfce_backdrop_timer(backdrop);
+}
+
 /* Generates the background that will either be displayed or will have the
  * image drawn on top of */
 static GdkPixbuf *
@@ -1410,10 +1424,6 @@ xfce_backdrop_image_data_release(XfceBackdropImageData *image_data)
 
     if(image_data->loader)
         g_object_unref(image_data->loader);
-
-    g_free(image_data);
-    image_data = NULL;
-
 }
 
 /**
@@ -1478,7 +1488,7 @@ xfce_backdrop_generate_async(XfceBackdrop *backdrop)
     else
         image_path = DEFAULT_BACKDROP;
 
-    DBG("loading image %s", image_path);
+    XF_DEBUG("loading image %s", image_path);
 
     file = g_file_new_for_path(image_path);
 
@@ -1584,6 +1594,7 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
     /* canceled? quit now */
     if(g_cancellable_is_cancelled(image_data->cancellable)) {
         xfce_backdrop_image_data_release(image_data);
+        g_free(image_data);
         return;
     }
 
@@ -1628,13 +1639,14 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
 
     /* no image and not canceled? return just the canvas */
     if(!image && !g_cancellable_is_cancelled(image_data->cancellable)) {
-        DBG("image failed to load, displaying canvas only");
+        XF_DEBUG("image failed to load, displaying canvas only");
         backdrop->priv->pix = final_image;
 
         g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_READY], 0);
 
         backdrop->priv->image_data = NULL;
         xfce_backdrop_image_data_release(image_data);
+        g_free(image_data);
         return;
     }
 
@@ -1732,6 +1744,7 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
 
     backdrop->priv->image_data = NULL;
     xfce_backdrop_image_data_release(image_data);
+    g_free(image_data);
 }
 
 static void
