@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,6 +31,7 @@
 
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
+#include <exo/exo.h>
 
 #include "xfdesktop-app-menu-item.h"
 
@@ -46,7 +46,7 @@ struct _XfdesktopAppMenuItem
 
 typedef struct _XfdesktopAppMenuItemClass
 {
-	GtkImageMenuItemClass parent;
+    GtkImageMenuItemClass parent;
 } XfdesktopAppMenuItemClass;
 
 enum
@@ -171,30 +171,31 @@ xfdesktop_app_menu_item_set_icon(XfdesktopAppMenuItem *app_menu_item)
     const gchar *icon_name;
     gint w, h, size;
     GdkPixbuf *pixbuf = NULL;
-    GtkWidget *image;
+    GtkWidget *image = NULL;
     GtkIconTheme *icon_theme;
     gchar *p, *name = NULL;
     gchar *filename;
 
     icon_name = garcon_menu_item_get_icon_name(app_menu_item->item);
+    icon_theme = gtk_icon_theme_get_default();
 
     if(G_LIKELY(icon_name)) {
         gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &w, &h);
         size = MIN(w, h);
 
-        if (g_path_is_absolute (icon_name)) {
-            pixbuf = gdk_pixbuf_new_from_file_at_scale(icon_name, w, h, TRUE, NULL);
-        } else {
-            icon_theme = gtk_icon_theme_get_default();
-            pixbuf = gtk_icon_theme_load_icon(icon_theme, icon_name, size, 0, NULL);
-
-            if (G_UNLIKELY(pixbuf == NULL)) {
+        if(gtk_icon_theme_has_icon(icon_theme, icon_name))
+            image = gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_MENU);
+        else {
+            if (g_path_is_absolute(icon_name)) {
+                pixbuf = gdk_pixbuf_new_from_file_at_scale(icon_name, w, h, TRUE, NULL);
+            } else {
                 /* try to lookup names like application.png in the theme */
                 p = strrchr(icon_name, '.');
                 if (p) {
                     name = g_strndup(icon_name, p - icon_name);
                     pixbuf = gtk_icon_theme_load_icon(icon_theme, name, size, 0, NULL);
                     g_free (name);
+                    name = NULL;
                 }
 
                 /* maybe they point to a file in the pixbufs folder */
@@ -202,21 +203,29 @@ xfdesktop_app_menu_item_set_icon(XfdesktopAppMenuItem *app_menu_item)
                     filename = g_build_filename("pixmaps", icon_name, NULL);
                     name = xfce_resource_lookup(XFCE_RESOURCE_DATA, filename);
                     g_free(filename);
+                }
 
-                    if(name)
-                        pixbuf = gdk_pixbuf_new_from_file(name, NULL);
+                if(name) {
+                    pixbuf = gdk_pixbuf_new_from_file_at_scale(name, w, h, TRUE, NULL);
                     g_free(name);
                 }
             }
+
+            /* Turn the pixbuf into a gtk_image */
+            if(G_LIKELY(pixbuf)) {
+                /* scale the pixbuf down if it needs it */
+                GdkPixbuf *tmp = exo_gdk_pixbuf_scale_down(pixbuf, TRUE, w, h);
+                g_object_unref(pixbuf);
+                pixbuf = tmp;
+
+                image = gtk_image_new_from_pixbuf(pixbuf);
+                g_object_unref(G_OBJECT(pixbuf));
+            }
         }
     }
 
-    if(G_LIKELY(pixbuf)) {
-        image = gtk_image_new_from_pixbuf(pixbuf);
-        g_object_unref(G_OBJECT(pixbuf));
-    } else {
+    if(!GTK_IS_IMAGE(image))
         image = gtk_image_new();
-    }
 
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(app_menu_item), image);
 }
@@ -225,9 +234,6 @@ static void
 xfdesktop_app_menu_item_changed(XfdesktopAppMenuItem *app_menu_item)
 {
     const gchar *label;
-#if !GTK_CHECK_VERSION (2, 16, 0)
-    GtkWidget   *child;
-#endif
 
     g_return_if_fail(XFCE_IS_APP_MENU_ITEM(app_menu_item));
     g_return_if_fail(GARCON_IS_MENU_ITEM(app_menu_item->item));
@@ -239,21 +245,7 @@ xfdesktop_app_menu_item_changed(XfdesktopAppMenuItem *app_menu_item)
     if (G_UNLIKELY (label == NULL))
       label = "";
 
-#if GTK_CHECK_VERSION(2, 16, 0)
     gtk_menu_item_set_label(GTK_MENU_ITEM(app_menu_item), label);
-#else
-    child = gtk_bin_get_child(GTK_BIN (app_menu_item));
-    if (child == NULL) {
-        child = gtk_accel_label_new(label);
-        gtk_container_add(GTK_CONTAINER(app_menu_item), child);
-        gtk_misc_set_alignment(GTK_MISC(child), 0.0, 0.5);
-        gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(child), GTK_WIDGET(app_menu_item));
-        gtk_widget_show(child);
-    } else {
-        g_return_if_fail(GTK_IS_LABEL(child));
-        gtk_label_set_text(GTK_LABEL(child), label);
-    }
-#endif
 }
 
 static void
@@ -339,16 +331,67 @@ xfdesktop_app_menu_item_command(XfdesktopAppMenuItem *app_menu_item)
     return g_string_free(newstr, FALSE);
 }
 
+static gboolean
+xfdesktop_app_menu_item_edit_launcher(XfdesktopAppMenuItem *app_menu_item)
+{
+    GFile *file;
+    gchar *uri, *cmd;
+    GError *error = NULL;
+    gboolean ret = FALSE;
+
+    file = garcon_menu_item_get_file(app_menu_item->item);
+
+    if(file) {
+        uri = g_file_get_uri(file);
+        cmd = g_strdup_printf("exo-desktop-item-edit \"%s\"", uri);
+
+        if(!xfce_spawn_command_line_on_screen(NULL, cmd, FALSE, FALSE, &error)) {
+            xfce_message_dialog(NULL, _("Launch Error"),
+                                GTK_STOCK_DIALOG_ERROR,
+                                _("Unable to launch \"exo-desktop-item-edit\", which is required to create and edit launchers and links on the desktop."),
+                                error->message, GTK_STOCK_CLOSE,
+                                GTK_RESPONSE_ACCEPT, NULL);
+            g_error_free(error);
+        } else {
+            ret = TRUE;
+        }
+
+        g_free(uri);
+        g_free(cmd);
+        g_object_unref(file);
+    }
+
+    return ret;
+}
+
 static void
 xfdesktop_app_menu_item_activate (XfdesktopAppMenuItem *app_menu_item)
 {
    gchar *command;
+   GdkEventButton *evt;
+   guint button;
    GError *error = NULL;
 
+   TRACE("entering");
+
    command = xfdesktop_app_menu_item_command(app_menu_item);
    if (command == NULL)
        return;
 
+   evt = (GdkEventButton *)gtk_get_current_event();
+
+   /* Right click edits the launchers */
+   if(evt && GDK_BUTTON_RELEASE == evt->type) {
+       button = evt->button;
+
+       if(button == 3 || (button == 1 && (evt->state & GDK_SHIFT_MASK))) {
+            if(xfdesktop_app_menu_item_edit_launcher(app_menu_item)) {
+                gdk_event_free((GdkEvent*)evt);
+                return;
+            }
+       }
+   }
+
    if(!xfce_spawn_command_line_on_screen(gtk_widget_get_screen(GTK_WIDGET(app_menu_item)),
                                          command,
                                          garcon_menu_item_requires_terminal(app_menu_item->item),
@@ -358,6 +401,8 @@ xfdesktop_app_menu_item_activate (XfdesktopAppMenuItem *app_menu_item)
                  command, error->message);
         g_error_free(error);
     }
+
+    gdk_event_free((GdkEvent*)evt);
 }
 
 GtkWidget *
