@@ -73,6 +73,10 @@ static void cb_xfdesktop_application_reload(GAction  *action,
                                             GVariant *parameter,
                                             gpointer  data);
 
+static void cb_xfdesktop_application_next(GAction  *action,
+                                          GVariant *parameter,
+                                          gpointer  data);
+
 static void xfdesktop_handle_quit_signals(gint sig, gpointer user_data);
 static void cb_xfdesktop_application_quit(GAction  *action,
                                           GVariant *parameter,
@@ -88,6 +92,10 @@ static void cb_xfdesktop_application_arrange(GAction  *action,
                                              GVariant *parameter,
                                              gpointer  data);
 
+static void cb_xfdesktop_application_debug(GAction  *action,
+                                           GVariant *parameter,
+                                           gpointer  data);
+
 static gboolean cb_wait_for_window_manager(gpointer data);
 static void     cb_wait_for_window_manager_destroyed(gpointer data);
 
@@ -182,6 +190,12 @@ xfdesktop_application_init(XfdesktopApplication *app)
     xfdesktop_application_add_action(app, G_ACTION(action));
     g_object_unref(action);
 
+    /* next action */
+    action = g_simple_action_new("next", NULL);
+    g_signal_connect(action, "activate", G_CALLBACK(cb_xfdesktop_application_next), app);
+    xfdesktop_application_add_action(app, G_ACTION(action));
+    g_object_unref(action);
+
     /* quit action */
     action = g_simple_action_new("quit", NULL);
     g_signal_connect(action, "activate", G_CALLBACK(cb_xfdesktop_application_quit), app);
@@ -200,6 +214,12 @@ xfdesktop_application_init(XfdesktopApplication *app)
     xfdesktop_application_add_action(app, G_ACTION(action));
     g_object_unref(action);
 
+    /* debug action, parameter toggles debug state */
+    action = g_simple_action_new("debug", G_VARIANT_TYPE_BOOLEAN);
+    g_signal_connect(action, "activate", G_CALLBACK(cb_xfdesktop_application_debug), app);
+    xfdesktop_application_add_action(app, G_ACTION(action));
+    g_object_unref(action);
+
 #if !GLIB_CHECK_VERSION (2, 32, 0)
     g_application_set_action_group(G_APPLICATION(app), (GActionGroup*)app->actions);
 #endif
@@ -357,7 +377,7 @@ reload_idle_cb(gpointer data)
     /* reload all the desktops */
     for(i = 0; i < app->nscreens; ++i) {
         if(app->desktops[i])
-            xfce_desktop_refresh(XFCE_DESKTOP(app->desktops[i]));
+            xfce_desktop_refresh(XFCE_DESKTOP(app->desktops[i]), FALSE);
     }
 
     menu_reload();
@@ -384,6 +404,31 @@ cb_xfdesktop_application_reload(GAction  *action,
     g_idle_add((GSourceFunc)reload_idle_cb, g_application);
 }
 
+static void
+cb_xfdesktop_application_next(GAction  *action,
+                              GVariant *parameter,
+                              gpointer  data)
+{
+    XfdesktopApplication *app;
+    gint i;
+
+    TRACE("entering");
+
+    g_return_if_fail(XFDESKTOP_IS_APPLICATION(data));
+
+    app = XFDESKTOP_APPLICATION(data);
+
+    /* If xfdesktop never started there's nothing to do here */
+    if(!app->desktops)
+        return;
+
+    /* reload all the desktops forcing the wallpaper to advance */
+    for(i = 0; i < app->nscreens; ++i) {
+        if(app->desktops[i])
+            xfce_desktop_refresh(XFCE_DESKTOP(app->desktops[i]), TRUE);
+    }
+}
+
 static void
 xfdesktop_handle_quit_signals(gint sig,
                               gpointer user_data)
@@ -497,6 +542,26 @@ cb_xfdesktop_application_arrange(GAction  *action,
     xfce_desktop_arrange_icons(XFCE_DESKTOP(app->desktops[screen_num]));
 }
 
+/* parameter is a boolean that determines whether to enable or disable the
+ * debug messages */
+static void
+cb_xfdesktop_application_debug(GAction  *action,
+                               GVariant *parameter,
+                               gpointer  data)
+{
+    TRACE("entering");
+
+    /* sanity checks */
+    if(!data || !XFDESKTOP_IS_APPLICATION(data))
+        return;
+
+    if(!g_variant_is_of_type(parameter, G_VARIANT_TYPE_BOOLEAN))
+        return;
+
+    /* Toggle the debug state */
+    xfdesktop_debug_set(g_variant_get_boolean(parameter));
+}
+
 /* Cleans up the associated wait for wm resources */
 static void
 wait_for_window_manager_cleanup(WaitForWM *wfwm)
@@ -523,7 +588,7 @@ cb_wait_for_window_manager(gpointer data)
 
     for(i = 0; i < wfwm->atom_count; i++) {
         if(XGetSelectionOwner(wfwm->dpy, wfwm->atoms[i]) == None) {
-            DBG("window manager not ready on screen %d", i);
+            XF_DEBUG("window manager not ready on screen %d", i);
             have_wm = FALSE;
             break;
         }
@@ -557,7 +622,7 @@ cb_wait_for_window_manager_destroyed(gpointer data)
         g_printerr("No window manager registered on screen 0. "
                    "To start the xfdesktop without this check, run with --disable-wm-check.\n");
     } else {
-        DBG("found window manager after %d tries", wfwm->counter);
+        XF_DEBUG("found window manager after %d tries", wfwm->counter);
     }
 
     /* start loading the desktop, hopefully a window manager is found, but it
@@ -759,19 +824,25 @@ xfdesktop_application_local_command_line(GApplication *g_application,
     GError *error = NULL;
     gboolean opt_version = FALSE;
     gboolean opt_reload = FALSE;
+    gboolean opt_next = FALSE;
     gboolean opt_menu = FALSE;
     gboolean opt_windowlist = FALSE;
     gboolean opt_arrange = FALSE;
     gboolean opt_quit = FALSE;
+    gboolean opt_enable_debug = FALSE;
+    gboolean opt_disable_debug = FALSE;
     gboolean option_set = FALSE;
     const GOptionEntry main_entries[] = {
         { "version", 'V', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_version, N_("Display version information"), NULL },
         { "reload", 'R', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_reload, N_("Reload all settings"), NULL },
+        { "next", 'N', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_next, N_("Advance to the next wallpaper on the current workspace"), NULL },
         { "menu", 'M', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_menu, N_("Pop up the menu (at the current mouse position)"), NULL },
         { "windowlist", 'W', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_windowlist, N_("Pop up the window list (at the current mouse position)"), NULL },
 #ifdef ENABLE_FILE_ICONS
         { "arrange", 'A', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_arrange, N_("Automatically arrange all the icons on the desktop"), NULL },
 #endif
+        { "enable-debug", 'e', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_enable_debug, N_("Enable debug messages"), NULL },
+        { "disable-debug", 'd', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_disable_debug, N_("Disable debug messages"), NULL },
         { "disable-wm-check", 'D', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &app->opt_disable_wm_check, N_("Do not wait for a window manager on startup"), NULL },
         { "quit", 'Q', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_quit, N_("Cause xfdesktop to quit"), NULL },
         { NULL, 0, 0, 0, NULL, NULL, NULL }
@@ -842,6 +913,9 @@ xfdesktop_application_local_command_line(GApplication *g_application,
     } else if(opt_reload) {
         g_action_group_activate_action(G_ACTION_GROUP(g_application), "reload", NULL);
         option_set = TRUE;
+    } else if(opt_next) {
+        g_action_group_activate_action(G_ACTION_GROUP(g_application), "next", NULL);
+        option_set = TRUE;
     } else if(opt_menu) {
         g_action_group_activate_action(G_ACTION_GROUP(g_application), "menu",
                                        g_variant_new_boolean(TRUE));
@@ -853,6 +927,14 @@ xfdesktop_application_local_command_line(GApplication *g_application,
     } else if(opt_arrange) {
         g_action_group_activate_action(G_ACTION_GROUP(g_application), "arrange", NULL);
         option_set = TRUE;
+    } else if(opt_enable_debug) {
+        g_action_group_activate_action(G_ACTION_GROUP(g_application), "debug",
+                                       g_variant_new_boolean(TRUE));
+        option_set = TRUE;
+    } else if(opt_disable_debug) {
+        g_action_group_activate_action(G_ACTION_GROUP(g_application), "debug",
+                                       g_variant_new_boolean(FALSE));
+        option_set = TRUE;
     }
 
     /* We handled the command line option */
