@@ -12,9 +12,9 @@
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  * Copied, renamed, and hacked to pieces by Brian Tarricone <bjt23@cornell.edu>.
  * Original code from Thunar.
@@ -78,11 +78,11 @@ static void xfdesktop_clipboard_manager_file_destroyed    (XfdesktopClipboardMan
 static void xfdesktop_clipboard_manager_owner_changed     (GtkClipboard                   *clipboard,
                                                            GdkEventOwnerChange            *event,
                                                            XfdesktopClipboardManager      *manager);
-#if 0
+
 static void xfdesktop_clipboard_manager_contents_received (GtkClipboard                   *clipboard,
                                                            GtkSelectionData               *selection_data,
                                                            gpointer                        user_data);
-#endif
+
 static void xfdesktop_clipboard_manager_targets_received  (GtkClipboard                   *clipboard,
                                                            GtkSelectionData               *selection_data,
                                                            gpointer                        user_data);
@@ -237,8 +237,12 @@ xfdesktop_clipboard_manager_finalize (GObject *object)
   g_list_free (manager->files);
 
   /* disconnect from the clipboard */
-  g_signal_handlers_disconnect_by_func (G_OBJECT (manager->clipboard), xfdesktop_clipboard_manager_owner_changed, manager);
-  g_object_set_qdata (G_OBJECT (manager->clipboard), xfdesktop_clipboard_manager_quark, NULL);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (manager->clipboard),
+                                        xfdesktop_clipboard_manager_owner_changed,
+                                        manager);
+  g_object_set_qdata (G_OBJECT (manager->clipboard),
+                      xfdesktop_clipboard_manager_quark,
+                      NULL);
   g_object_unref (G_OBJECT (manager->clipboard));
 
   (*G_OBJECT_CLASS (xfdesktop_clipboard_manager_parent_class)->finalize) (object);
@@ -307,7 +311,6 @@ xfdesktop_clipboard_manager_owner_changed (GtkClipboard           *clipboard,
 }
 
 
-#if 0
 static void
 xfdesktop_clipboard_manager_contents_received (GtkClipboard     *clipboard,
                                             GtkSelectionData *selection_data,
@@ -318,14 +321,16 @@ xfdesktop_clipboard_manager_contents_received (GtkClipboard     *clipboard,
   GtkWindow                      *parent = GTK_WINDOW(gtk_widget_get_toplevel(request->widget));
   gboolean                        path_copy = TRUE;
   GList                          *path_list = NULL;
+  GList                          *dest_file_list  = NULL;
+  GList                          *l               = NULL;
   gchar                          *data;
 
   /* check whether the retrieval worked */
-  if (G_LIKELY (selection_data->length > 0))
+  if (G_LIKELY (gtk_selection_data_get_length(selection_data) > 0))
     {
       /* be sure the selection data is zero-terminated */
-      data = (gchar *) selection_data->data;
-      data[selection_data->length] = '\0';
+      data = (gchar *) gtk_selection_data_get_data(selection_data);
+      data[gtk_selection_data_get_length(selection_data)] = '\0';
 
       /* check whether to copy or move */
       if (g_ascii_strncasecmp (data, "copy\n", 5) == 0)
@@ -340,19 +345,43 @@ xfdesktop_clipboard_manager_contents_received (GtkClipboard     *clipboard,
         }
 
       /* determine the path list stored with the selection */
-      path_list = thunar_vfs_path_list_from_string (data, NULL);
+      path_list = xfdesktop_file_utils_file_list_from_string (data);
     }
 
   /* perform the action if possible */
   if (G_LIKELY (path_list != NULL))
     {
+      for (l = path_list; l; l = l->next) {
+        gchar *dest_basename = g_file_get_basename(l->data);
+
+        if(dest_basename && *dest_basename != '\0') {
+          /* If we copy a file, we need to use the new absolute filename
+           * as the destination. If we move, we need to use the destination
+           * directory. */
+           if(path_copy) {
+             GFile *dest_file = g_file_get_child(request->target_file, dest_basename);
+             dest_file_list = g_list_prepend(dest_file_list, dest_file);
+           } else {
+             dest_file_list = g_list_prepend(dest_file_list, request->target_file);
+           }
+         }
+         g_free(dest_basename);
+      }
+
+      dest_file_list = g_list_reverse(dest_file_list);
+
       if (G_LIKELY (path_copy))
-        xfdesktop_file_utils_copy_into(parent, path_list, request->target_path);
-        //thunar_application_copy_into (application, request->widget, path_list, request->target_path, request->new_files_closure);
-      else
-        xfdesktop_file_utils_move_into(parent, path_list, request->target_path);
-        //thunar_application_move_into (application, request->widget, path_list, request->target_path, request->new_files_closure);
-      thunar_vfs_path_list_free (path_list);
+      {
+        xfdesktop_file_utils_transfer_files(GDK_ACTION_COPY,
+                                            path_list,
+                                            dest_file_list,
+                                            gtk_widget_get_screen(GTK_WIDGET(parent)));
+      } else {
+        xfdesktop_file_utils_transfer_files(GDK_ACTION_MOVE,
+                                            path_list,
+                                            dest_file_list,
+                                            gtk_widget_get_screen(GTK_WIDGET(parent)));
+      }
 
       /* clear the clipboard if it contained "cutted data"
        * (gtk_clipboard_clear takes care of not clearing
@@ -365,18 +394,11 @@ xfdesktop_clipboard_manager_contents_received (GtkClipboard     *clipboard,
        * if either the Xserver or our GTK+ version
        * doesn't support the XFixes extension.
        */
-#if GTK_CHECK_VERSION(2,6,0)
       if (!gdk_display_supports_selection_notification (gtk_clipboard_get_display (manager->clipboard)))
-#endif
         {
           xfdesktop_clipboard_manager_owner_changed (manager->clipboard, NULL, manager);
         }
     }
-  else
-    {
-      /* tell the user that we cannot paste */
-//      thunar_dialogs_show_error (request->widget, NULL, _("There is nothing on the clipboard to paste"));
-    }
 
   /* free the request */
   if (G_LIKELY (request->widget != NULL))
@@ -384,10 +406,11 @@ xfdesktop_clipboard_manager_contents_received (GtkClipboard     *clipboard,
   if (G_LIKELY (request->new_files_closure != NULL))
     g_closure_unref (request->new_files_closure);
   g_object_unref (G_OBJECT (request->manager));
-  thunar_vfs_path_unref (request->target_path);
-  g_free (request);
+
+  g_list_free(dest_file_list);
+  g_list_free(path_list);
 }
-#endif
+
 
 
 static void
@@ -454,12 +477,20 @@ xfdesktop_clipboard_manager_get_callback (GtkClipboard     *clipboard,
     {
     case TARGET_GNOME_COPIED_FILES:
       data = g_strconcat (manager->files_cutted ? "cut\n" : "copy\n", string_list, NULL);
-      gtk_selection_data_set (selection_data, selection_data->target, 8, (guchar *) data, strlen (data));
+      gtk_selection_data_set (selection_data,
+                              gtk_selection_data_get_target(selection_data),
+                              8,
+                              (guchar *) data,
+                              strlen (data));
       g_free (data);
       break;
 
     case TARGET_UTF8_STRING:
-      gtk_selection_data_set (selection_data, selection_data->target, 8, (guchar *) string_list, strlen (string_list));
+      gtk_selection_data_set (selection_data,
+                              gtk_selection_data_get_target(selection_data),
+                              8,
+                              (guchar *) string_list,
+                              strlen (string_list));
       break;
 
     default:
@@ -654,6 +685,61 @@ xfdesktop_clipboard_manager_cut_files (XfdesktopClipboardManager *manager,
 gboolean
 xfdesktop_clipboard_manager_get_can_paste (XfdesktopClipboardManager *manager)
 {
-    /* FIXME: implement */
-    return FALSE;
+    g_return_val_if_fail (XFDESKTOP_IS_CLIPBOARD_MANAGER (manager), FALSE);
+    return manager->can_paste;
+}
+
+
+/**
+ * thunar_clipboard_manager_paste_files:
+ * @manager           : a #XfdesktopClipboardManager.
+ * @target_file       : the #GFile of the folder to which the contents on the clipboard
+ *                      should be pasted.
+ * @widget            : a #GtkWidget, on which to perform the paste or %NULL if no widget is
+ *                      known.
+ * @new_files_closure : a #GClosure to connect to the job's "new-files" signal,
+ *                      which will be emitted when the job finishes with the
+ *                      list of #GFile<!---->s created by the job, or
+ *                      %NULL if you're not interested in the signal.
+ *
+ * Pastes the contents from the clipboard associated with @manager to the directory
+ * referenced by @target_file.
+ * Code copied and adapted from thunar-clipboard-manager.c
+ * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009-2011 Jannis Pohlmann <jannis@xfce.org>
+ **/
+void
+xfdesktop_clipboard_manager_paste_files (XfdesktopClipboardManager *manager,
+                                      GFile                  *target_file,
+                                      GtkWidget              *widget,
+                                      GClosure               *new_files_closure)
+{
+  XfdesktopClipboardPasteRequest *request;
+
+  g_return_if_fail (XFDESKTOP_IS_CLIPBOARD_MANAGER (manager));
+  g_return_if_fail (widget == NULL || GTK_IS_WIDGET (widget));
+
+  /* prepare the paste request */
+  request = g_slice_new0 (XfdesktopClipboardPasteRequest);
+  request->manager = g_object_ref (G_OBJECT (manager));
+  request->target_file = g_object_ref (target_file);
+  request->widget = widget;
+
+  /* take a reference on the closure (if any) */
+  if (G_LIKELY (new_files_closure != NULL))
+    {
+      request->new_files_closure = new_files_closure;
+      g_closure_ref (new_files_closure);
+      g_closure_sink (new_files_closure);
+    }
+
+  /* get notified when the widget is destroyed prior to
+   * completing the clipboard contents retrieval
+   */
+  if (G_LIKELY (request->widget != NULL))
+    g_object_add_weak_pointer (G_OBJECT (request->widget), (gpointer) &request->widget);
+
+  /* schedule the request */
+  gtk_clipboard_request_contents (manager->clipboard, manager->x_special_gnome_copied_files,
+                                  xfdesktop_clipboard_manager_contents_received, request);
 }
