@@ -48,6 +48,9 @@ struct _XfdesktopFileIconManager
 struct _XfdesktopFileIconManagerClass
 {
     GObjectClass parent;
+
+    /*< signals >*/
+    void (*hidden_state_changed)(XfdesktopFileIconManager *fmanager);
 };
 
 GType xfdesktop_file_icon_manager_get_type(void) G_GNUC_CONST;
@@ -55,6 +58,8 @@ GType xfdesktop_file_icon_manager_get_type(void) G_GNUC_CONST;
 XfdesktopIconViewManager *xfdesktop_file_icon_manager_new(GFile *folder,
                                                           XfconfChannel *channel);
 
+void xfdesktop_file_icon_save(gpointer user_data);
+
 gboolean xfdesktop_file_icon_manager_get_cached_icon_position(
                                                     XfdesktopFileIconManager *fmanager,
                                                     const gchar *name,
