@@ -781,47 +781,11 @@ xfdesktop_file_utils_bulk_rename(GFile *working_directory,
     }
 }
 
-#if GLIB_CHECK_VERSION (2, 38, 0)
-static void
-delete_files(gpointer data, gpointer user_data)
-{
-    if(data == NULL)
-        return;
-
-    if(!G_IS_FILE(data))
-        return;
-
-    g_file_delete_async(G_FILE(data), G_PRIORITY_DEFAULT, NULL, NULL, NULL);
-}
-#endif
-
-#if GLIB_CHECK_VERSION (2, 38, 0)
-static void
-trash_files(gpointer data, gpointer user_data)
-{
-    if(data == NULL)
-        return;
-
-    if(!G_IS_FILE(data))
-        return;
-
-    g_file_trash_async(G_FILE(data), G_PRIORITY_DEFAULT, NULL, NULL, NULL);
-}
-#endif
-
 void
 xfdesktop_file_utils_unlink_files(GList *files,
                                   GdkScreen *screen,
                                   GtkWindow *parent)
 {
-#if GLIB_CHECK_VERSION (2, 38, 0)
-    g_return_if_fail(files != NULL && G_IS_FILE(files->data));
-
-    g_list_foreach(files, (GFunc)delete_files, NULL);
-    /* return statement to explicitly show we don't
-     * want to run any of the fallback code. */
-    return;
-#else
     DBusGProxy *fileman_proxy;
 
     g_return_if_fail(files != NULL && G_IS_FILE(files->data));
@@ -865,7 +829,6 @@ xfdesktop_file_utils_unlink_files(GList *files,
                               "be present (such as the one supplied by Thunar)."),
                             GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
     }
-#endif
 }
 
 void
@@ -873,14 +836,6 @@ xfdesktop_file_utils_trash_files(GList *files,
                                  GdkScreen *screen,
                                  GtkWindow *parent)
 {
-#if GLIB_CHECK_VERSION (2, 38, 0)
-    g_return_if_fail(files != NULL && G_IS_FILE(files->data));
-
-    g_list_foreach(files, (GFunc)trash_files, NULL);
-    /* return statement to explicitly show we don't
-     * want to run any of the fallback code. */
-    return;
-#else
     DBusGProxy *trash_proxy;
 
     g_return_if_fail(files != NULL && G_IS_FILE(files->data));
@@ -924,7 +879,6 @@ xfdesktop_file_utils_trash_files(GList *files,
                               "be present (such as the one supplied by Thunar)."),
                             GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
     }
-#endif
 }
 
 void
