@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifndef __XFDESKTOP_WINDOW_ICON_MANAGER_H__
@@ -49,7 +49,7 @@ struct _XfdesktopWindowIconManagerClass
     GObjectClass parent;
 };
 
-GType xfdesktop_window_icon_manager_get_type() G_GNUC_CONST;
+GType xfdesktop_window_icon_manager_get_type(void) G_GNUC_CONST;
 
 XfdesktopIconViewManager *xfdesktop_window_icon_manager_new(GdkScreen *gscreen);
 
