@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,34 +26,33 @@
 #include <string.h>
 #endif
 
-#include <libxfcegui4/libxfcegui4.h>
-#include <libxfcegui4/netk-window-action-menu.h>
+#include <libwnck/libwnck.h>
+#include <libwnck/window-action-menu.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include "xfdesktop-window-icon.h"
 
 struct _XfdesktopWindowIconPrivate
 {
     gint workspace;
-    GdkPixbuf *pix;
-    gint cur_pix_size;
     gchar *label;
-    NetkWindow *window;
+    WnckWindow *window;
 };
 
-static void xfdesktop_window_icon_class_init(XfdesktopWindowIconClass *klass);
-static void xfdesktop_window_icon_init(XfdesktopWindowIcon *icon);
 static void xfdesktop_window_icon_finalize(GObject *obj);
 
 static GdkPixbuf *xfdesktop_window_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                                   gint size);
-static G_CONST_RETURN gchar *xfdesktop_window_icon_peek_label(XfdesktopIcon *icon);
+                                                   gint width, gint height);
+static const gchar *xfdesktop_window_icon_peek_label(XfdesktopIcon *icon);
+static gchar *xfdesktop_window_icon_get_identifier(XfdesktopIcon *icon);
 
 static gboolean xfdesktop_window_icon_activated(XfdesktopIcon *icon);
-static void xfdesktop_window_icon_menu_popup(XfdesktopIcon *icon);
+static gboolean xfdesktop_window_icon_populate_context_menu(XfdesktopIcon *icon,
+                                                            GtkWidget *menu);
 
-static void xfdesktop_window_name_changed_cb(NetkWindow *window,
+static void xfdesktop_window_name_changed_cb(WnckWindow *window,
                                              gpointer user_data);
-static void xfdesktop_window_icon_changed_cb(NetkWindow *window,
+static void xfdesktop_window_icon_changed_cb(WnckWindow *window,
                                              gpointer user_data);
 
 
@@ -72,8 +71,9 @@ xfdesktop_window_icon_class_init(XfdesktopWindowIconClass *klass)
     
     icon_class->peek_pixbuf = xfdesktop_window_icon_peek_pixbuf;
     icon_class->peek_label = xfdesktop_window_icon_peek_label;
+    icon_class->get_identifier = xfdesktop_window_icon_get_identifier;
     icon_class->activated = xfdesktop_window_icon_activated;
-    icon_class->menu_popup = xfdesktop_window_icon_menu_popup;
+    icon_class->populate_context_menu = xfdesktop_window_icon_populate_context_menu;
 }
 
 static void
@@ -90,9 +90,6 @@ xfdesktop_window_icon_finalize(GObject *obj)
     gchar data_name[256];
     guint16 row, col;
     
-    if(icon->priv->pix)
-        g_object_unref(G_OBJECT(icon->priv->pix));
-    
     g_free(icon->priv->label);
     
     /* save previous position */
@@ -120,7 +117,7 @@ xfdesktop_window_icon_finalize(GObject *obj)
 
 
 static void
-xfdesktop_window_name_changed_cb(NetkWindow *window,
+xfdesktop_window_name_changed_cb(WnckWindow *window,
                                  gpointer user_data)
 {
     XfdesktopWindowIcon *icon = user_data;
@@ -132,108 +129,81 @@ xfdesktop_window_name_changed_cb(NetkWindow *window,
 }
 
 static void
-xfdesktop_window_icon_changed_cb(NetkWindow *window,
+xfdesktop_window_icon_changed_cb(WnckWindow *window,
                                  gpointer user_data)
 {
     XfdesktopWindowIcon *icon = user_data;
-    
-    if(icon->priv->pix) {
-        g_object_unref(G_OBJECT(icon->priv->pix));
-        icon->priv->pix = NULL;
-    }
-    
+
+    xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(icon));
     xfdesktop_icon_label_changed(XFDESKTOP_ICON(icon));
 }
 
-
-
-
-
 static GdkPixbuf *
 xfdesktop_window_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                 gint size)
+                                 gint width, gint height)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
-    
-    if(!window_icon->priv->pix || window_icon->priv->cur_pix_size != size) {
-        if(window_icon->priv->pix)
-            g_object_unref(G_OBJECT(window_icon->priv->pix));
-        
-        window_icon->priv->pix = netk_window_get_icon(window_icon->priv->window);
-        if(window_icon->priv->pix) {
-            if(gdk_pixbuf_get_width(window_icon->priv->pix) != size) {
-                window_icon->priv->pix = gdk_pixbuf_scale_simple(window_icon->priv->pix,
-                                                                 size,
-                                                                 size,
-                                                                 GDK_INTERP_BILINEAR);
-            } else
-                g_object_ref(G_OBJECT(window_icon->priv->pix));
-            window_icon->priv->cur_pix_size = size;
-        }
+    GdkPixbuf *pix = NULL;
+
+    pix = wnck_window_get_icon(window_icon->priv->window);
+    if(pix) {
+        if(gdk_pixbuf_get_height(pix) != height) {
+            pix = gdk_pixbuf_scale_simple(pix, height, height, GDK_INTERP_BILINEAR);
+        } else
+            g_object_ref(G_OBJECT(pix));
     }
-    
-    return window_icon->priv->pix;
+
+    return pix;
 }
 
-static G_CONST_RETURN gchar *
+static const gchar *
 xfdesktop_window_icon_peek_label(XfdesktopIcon *icon)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
     
     if(!window_icon->priv->label)
-        window_icon->priv->label = g_strdup(netk_window_get_name(window_icon->priv->window));
+        window_icon->priv->label = g_strdup(wnck_window_get_name(window_icon->priv->window));
     
     return window_icon->priv->label;
 }
 
+static gchar *
+xfdesktop_window_icon_get_identifier(XfdesktopIcon *icon)
+{
+    return g_strdup(xfdesktop_window_icon_peek_label(icon));
+}
+
 static gboolean
 xfdesktop_window_icon_activated(XfdesktopIcon *icon)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
     
-    netk_window_activate(window_icon->priv->window);
+    wnck_window_activate(window_icon->priv->window,
+                         gtk_get_current_event_time());
     
     return TRUE;
 }
 
 static gboolean
-xfdesktop_action_menu_destroy_idled(gpointer data)
-{
-    gtk_widget_destroy(GTK_WIDGET(data));
-    return FALSE;
-}
-
-static void
-xfdesktop_action_menu_deactivate_cb(GtkMenu *menu,
-                          gpointer user_data)
-{
-    g_idle_add(xfdesktop_action_menu_destroy_idled, menu);
-}
-
-static void
-xfdesktop_window_icon_menu_popup(XfdesktopIcon *icon)
+xfdesktop_window_icon_populate_context_menu(XfdesktopIcon *icon,
+                                            GtkWidget *menu)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
-    GtkWidget *menu = netk_create_window_action_menu(window_icon->priv->window);
-    NetkScreen *netk_screen;
-    gint screen_num;
-    GdkScreen *gscreen;
-    
-    netk_screen = netk_window_get_screen(window_icon->priv->window);
-    screen_num = netk_screen_get_number(netk_screen);
-    gscreen = gdk_display_get_screen(gdk_display_get_default(), screen_num);
-    gtk_menu_set_screen(GTK_MENU(menu), gscreen);
-    gtk_widget_show(menu);
-    g_signal_connect(G_OBJECT(menu), "deactivate",
-                     G_CALLBACK(xfdesktop_action_menu_deactivate_cb), NULL);
-    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
-                   3, gtk_get_current_event_time());
+    GtkWidget *amenu = wnck_create_window_action_menu(window_icon->priv->window);
+    GtkWidget *mi;
+
+    mi = gtk_menu_item_new_with_mnemonic(_("_Window Actions"));
+    gtk_menu_item_set_submenu (GTK_MENU_ITEM(mi), amenu);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+    gtk_widget_show_all(mi);
+
+    return TRUE;
 }
 
 
 
 XfdesktopWindowIcon *
-xfdesktop_window_icon_new(NetkWindow *window,
+xfdesktop_window_icon_new(WnckWindow *window,
                           gint workspace)
 {
     XfdesktopWindowIcon *icon = g_object_new(XFDESKTOP_TYPE_WINDOW_ICON, NULL);
